def get_param(key, default_value=None, default_config_path='./config.json', local_config_path="./local-config.json"):
    """
    读取优先级: 环境变量 > 本地配置(./local-config.json) > 默认配置(./config.json) > 默认值(default_value)
    """
    import os
    from .zfile import read_file
    import json

    # 从环境变量读取
    value = os.environ.get(key)
    if value is not None:
        return value

    # 从本地配置读取
    try:
        config = json.loads(read_file(local_config_path))
        value = config.get(key)
        if value is not None:
            return value
    except Exception as ex:
        pass

    # 从默认配置配置读取
    try:
        config = json.loads(read_file(default_config_path))
        value = config.get(key)
        if value is not None:
            return value
    except Exception as ex:
        pass

    # 返回默认值
    return default_value



import unittest


class TestCase(unittest.TestCase):
    def test_oss_config(self):
        from zzpy import OssConfig
        url = "oss://zeros.top/b-bucket?key=kkey&secret=ssecret"
        config = OssConfig(url)
        self.assertEqual(config.endpoint, f"https://zeros.top")
        self.assertEqual(config.bucket, "b-bucket")
        self.assertEqual(config.access_key_id, "kkey")
        self.assertEqual(config.access_key_secret, "ssecret")



def get_date(days=0, separator='-'):
    import datetime
    return (datetime.datetime.now() + datetime.timedelta(days=days)).strftime(f"%Y{separator}%m{separator}%d")


def get_today(separator='-'):
    return get_date(days=0, separator=separator)


def get_month(months=0, separator='-'):
    """获取月份，如2019-10"""
    import arrow
    return arrow.now().shift(months=months).format(f"YYYY{separator}MM")



import unittest


class ZRedisTests(unittest.TestCase):
    def setUp(self):
        import os
        super().setUp()
        self.root_dir = os.path.join("test", "file")
        from zzpy import remove_dir
        remove_dir(self.root_dir)

    def tearDown(self):
        from zzpy import remove_dir
        super().tearDown()
        remove_dir(self.root_dir)

    def test_redis_decode(self):
        from zzpy import redis_decode

        self.assertEqual(redis_decode(b"abc"), u"abc")
        self.assertEqual(redis_decode("你好".encode("utf8")), u"你好")
        self.assertEqual(redis_decode([b"1", "2", 3, u"x".encode("gbk"), u"再见".encode("utf8")]), [
                         u"1", u"2", 3, u"x", u"再见"])
        self.assertEqual(redis_decode(
            (b"1", "2", 3, u"再见".encode("utf8"))), (u"1", u"2", 3, u"再见"))
        self.assertEqual(redis_decode(
            {b"1", "2", 3, u"再见".encode("utf8")}), {u"1", u"2", 3, u"再见"})



import unittest


class ZMySQLTests(unittest.TestCase):
    def setUp(self):
        import os
        super().setUp()
        self.root_dir = os.path.join("test", "file")
        from zzpy import remove_dir
        remove_dir(self.root_dir)

    def tearDown(self):
        from zzpy import remove_dir
        super().tearDown()
        remove_dir(self.root_dir)

    def test_MySQLConfig(self):
        from zzpy import MySQLConfig

        self.assertEqual(MySQLConfig(url="mysql://1.2.3.4"),
                         MySQLConfig(host="1.2.3.4", port=3306))
        self.assertEqual(MySQLConfig(url="mysql://1.2.3.4/"),
                         MySQLConfig(host="1.2.3.4", port=3306))
        self.assertEqual(MySQLConfig(url="mysql://1.2.3.4:5"),
                         MySQLConfig(host="1.2.3.4", port=5))
        self.assertEqual(MySQLConfig(url="mysql://1.2.3.4:5/d"),
                         MySQLConfig(host="1.2.3.4", port=5, database="d"))
        self.assertEqual(MySQLConfig(url="mysql://1.2.3.4/d"),
                         MySQLConfig(host="1.2.3.4", port=3306, database="d"))
        self.assertEqual(MySQLConfig(url="mysql://1.2.3.4/d?"),
                         MySQLConfig(host="1.2.3.4", port=3306, database="d"))
        self.assertEqual(MySQLConfig(url="mysql://1.2.3.4:5/d?x=1"),
                         MySQLConfig(host="1.2.3.4", port=5, database="d", param={"x": "1"}))
        self.assertEqual(MySQLConfig(url="mysql://1.2.3.4:5/d?x=1&y=2"),
                         MySQLConfig(host="1.2.3.4", port=5, database="d", param={"x": "1", "y": "2"}))

        self.assertEqual(MySQLConfig(url="mysql://localhost"),
                         MySQLConfig(host="localhost", port=3306))
        self.assertEqual(MySQLConfig(url="mysql://localhost/"),
                         MySQLConfig(host="localhost", port=3306))
        self.assertEqual(MySQLConfig(url="mysql://localhost:5"),
                         MySQLConfig(host="localhost", port=5))
        self.assertEqual(MySQLConfig(url="mysql://localhost:5/d"),
                         MySQLConfig(host="localhost", port=5, database="d"))
        self.assertEqual(MySQLConfig(url="mysql://localhost/d"),
                         MySQLConfig(host="localhost", port=3306, database="d"))
        self.assertEqual(MySQLConfig(url="mysql://localhost/d?"),
                         MySQLConfig(host="localhost", port=3306, database="d"))
        self.assertEqual(MySQLConfig(url="mysql://localhost:5/d?x=1"),
                         MySQLConfig(host="localhost", port=5, database="d", param={"x": "1"}))
        self.assertEqual(MySQLConfig(url="mysql://localhost:5/d?x=1&y=2"),
                         MySQLConfig(host="localhost", port=5, database="d", param={"x": "1", "y": "2"}))

        self.assertEqual(MySQLConfig(url="mysql://a"),
                         MySQLConfig(host="a", port=3306))
        self.assertEqual(MySQLConfig(url="mysql://a/"),
                         MySQLConfig(host="a", port=3306))
        self.assertEqual(MySQLConfig(url="mysql://a:5"),
                         MySQLConfig(host="a", port=5))
        self.assertEqual(MySQLConfig(url="mysql://a:5/d"),
                         MySQLConfig(host="a", port=5, database="d"))
        self.assertEqual(MySQLConfig(url="mysql://a/d"),
                         MySQLConfig(host="a", port=3306, database="d"))
        self.assertEqual(MySQLConfig(url="mysql://a/d?"),
                         MySQLConfig(host="a", port=3306, database="d"))
        self.assertEqual(MySQLConfig(url="mysql://a:5/d?x=1"),
                         MySQLConfig(host="a", port=5, database="d", param={"x": "1"}))
        self.assertEqual(MySQLConfig(url="mysql://a:5/d?x=1&y=2"),
                         MySQLConfig(host="a", port=5, database="d", param={"x": "1", "y": "2"}))

        self.assertEqual(MySQLConfig(url="1.2.3.4"),
                         MySQLConfig(host="1.2.3.4", port=3306))
        self.assertEqual(MySQLConfig(url="1.2.3.4/"),
                         MySQLConfig(host="1.2.3.4", port=3306))
        self.assertEqual(MySQLConfig(url="1.2.3.4:5"),
                         MySQLConfig(host="1.2.3.4", port=5))
        self.assertEqual(MySQLConfig(url="1.2.3.4:5/d"),
                         MySQLConfig(host="1.2.3.4", port=5, database="d"))
        self.assertEqual(MySQLConfig(url="1.2.3.4/d"),
                         MySQLConfig(host="1.2.3.4", port=3306, database="d"))
        self.assertEqual(MySQLConfig(url="1.2.3.4/d?"),
                         MySQLConfig(host="1.2.3.4", port=3306, database="d"))
        self.assertEqual(MySQLConfig(url="1.2.3.4:5/d?x=1"),
                         MySQLConfig(host="1.2.3.4", port=5, database="d", param={"x": "1"}))
        self.assertEqual(MySQLConfig(url="1.2.3.4:5/d?x=1&y=2"),
                         MySQLConfig(host="1.2.3.4", port=5, database="d", param={"x": "1", "y": "2"}))

        self.assertEqual(MySQLConfig(url="localhost"),
                         MySQLConfig(host="localhost", port=3306))
        self.assertEqual(MySQLConfig(url="localhost/"),
                         MySQLConfig(host="localhost", port=3306))
        self.assertEqual(MySQLConfig(url="localhost:5"),
                         MySQLConfig(host="localhost", port=5))
        self.assertEqual(MySQLConfig(url="localhost:5/d"),
                         MySQLConfig(host="localhost", port=5, database="d"))
        self.assertEqual(MySQLConfig(url="localhost/d"),
                         MySQLConfig(host="localhost", port=3306, database="d"))
        self.assertEqual(MySQLConfig(url="localhost/d?"),
                         MySQLConfig(host="localhost", port=3306, database="d"))
        self.assertEqual(MySQLConfig(url="localhost:5/d?x=1"),
                         MySQLConfig(host="localhost", port=5, database="d", param={"x": "1"}))
        self.assertEqual(MySQLConfig(url="localhost:5/d?x=1&y=2"),
                         MySQLConfig(host="localhost", port=5, database="d", param={"x": "1", "y": "2"}))

        self.assertEqual(MySQLConfig(url="a"),
                         MySQLConfig(host="a", port=3306))
        self.assertEqual(MySQLConfig(url="a/"),
                         MySQLConfig(host="a", port=3306))
        self.assertEqual(MySQLConfig(url="a:5"),
                         MySQLConfig(host="a", port=5))
        self.assertEqual(MySQLConfig(url="a:5/d"),
                         MySQLConfig(host="a", port=5, database="d"))
        self.assertEqual(MySQLConfig(url="a/d"),
                         MySQLConfig(host="a", port=3306, database="d"))
        self.assertEqual(MySQLConfig(url="a/d?"),
                         MySQLConfig(host="a", port=3306, database="d"))
        self.assertEqual(MySQLConfig(url="a:5/d?x=1"),
                         MySQLConfig(host="a", port=5, database="d", param={"x": "1"}))
        self.assertEqual(MySQLConfig(url="a:5/d?x=1&y=2"),
                         MySQLConfig(host="a", port=5, database="d", param={"x": "1", "y": "2"}))

        self.assertEqual(MySQLConfig(url="mysql://a.b.c:123/?user=uuuu&password=pppp").to_dict(), {
                         "host": "a.b.c", "port": 123, "user": "uuuu", "password": "pppp", "param": {"user": "uuuu", "password": "pppp"}, "url": "mysql://a.b.c:123/?user=uuuu&password=pppp"})



import unittest


class ZfileTests(unittest.TestCase):
    def setUp(self):
        import os
        super().setUp()
        self.root_dir = os.path.join("test", "file")
        from zzpy import remove_dir
        remove_dir(self.root_dir)

    def tearDown(self):
        from zzpy import remove_dir
        super().tearDown()
        remove_dir(self.root_dir)

    def test_read_jsonline(self):
        from zzpy import read_jsonline
        from collections.abc import Generator
        import os

        # generator
        file_path = os.path.join("test", "static", "1.jsonl")
        self.assertIsInstance(read_jsonline(file_path), Generator)

        # end file without newline
        data = []
        for i in read_jsonline(file_path):
            data.append(i)
        self.assertListEqual(data, [{"name": "Zero", "age": 31},
                                    {"name": "Flyoung", "age": 17}])

        # generator
        file_path = os.path.join("test", "static", "2.jsonl")
        self.assertIsInstance(read_jsonline(file_path), Generator)

        # end file with newline
        data = []
        for i in read_jsonline(file_path):
            data.append(i)
        self.assertListEqual(data, [{"name": "Zero", "age": 31},
                                    {"name": "Flyoung", "age": 17}])

    def test_create_and_remove_dir(self):
        """测试文件夹的创建和删除"""
        from zzpy import create_dir, remove_dir, init_dir
        from zzpy import get_file_name_list_from_dir
        import os

        # ending without "/"
        a_dir = os.path.join(self.root_dir, "a")
        b_dir = os.path.join(self.root_dir, "a", "b")

        create_dir(b_dir)
        self.assertTrue(os.path.exists(b_dir))

        init_dir(a_dir)
        self.assertTrue(os.path.exists(a_dir))
        self.assertFalse(os.path.exists(b_dir))

    def test_create_and_remove_dir_ending_with_slash(self):
        """测试文件夹名最后带斜线/的创建和删除"""
        from zzpy import create_dir, remove_dir, init_dir
        import os

        # ending with "/"
        a_dir = os.path.join(self.root_dir, "a/")
        b_dir = os.path.join(self.root_dir, "a", "b/")

        create_dir(b_dir)
        self.assertTrue(os.path.exists(b_dir))

        init_dir(a_dir)
        self.assertTrue(os.path.exists(a_dir))
        self.assertFalse(os.path.exists(b_dir))

    def test_init_dirs(self):
        from zzpy import init_dirs
        from zzpy import get_tmp_dir_path
        from zzpy import get_work_dir_path
        from zzpy import remove_dir
        from zzpy import get_file_name_list_from_dir
        import os

        tmp_dir = get_tmp_dir_path(root_dir=self.root_dir)
        work_dir = get_work_dir_path(root_dir=self.root_dir)

        self.assertFalse(os.path.exists(self.root_dir))
        init_dirs(root_dir=self.root_dir)
        self.assertTrue(os.path.exists(self.root_dir))
        self.assertTrue(os.path.exists(tmp_dir))
        self.assertTrue(os.path.exists(work_dir))

    def test_get_file_path_list_from_dir(self):
        from zzpy import get_file_path_list_from_dir
        from zzpy import write_file
        from zzpy import init_dir
        import os

        # init
        a_dir = os.path.join(self.root_dir, "a")
        aa_dir = os.path.join(a_dir, "aa")
        init_dir(aa_dir)
        a10_path = os.path.join(a_dir, "10.txt")
        write_file("10.txt", a10_path)
        aa11_path = os.path.join(aa_dir, "11.txt")
        write_file("11.txt", aa11_path)

        b_dir = os.path.join(self.root_dir, "b")
        bb_dir = os.path.join(b_dir, "bb")
        init_dir(bb_dir)
        bb20_path = os.path.join(bb_dir, "20.txt")
        write_file("20.txt", bb20_path)
        bb21_path = os.path.join(bb_dir, "21.txt")
        write_file("21.txt", bb21_path)

        c_dir = os.path.join(self.root_dir, "c")
        init_dir(c_dir)
        c30_path = os.path.join(c_dir, "30.txt")
        write_file("30.txt", c30_path)

        r_path = os.path.join(self.root_dir, "r.txt")
        write_file("r.txt", r_path)

        # test
        file_path_list = get_file_path_list_from_dir(
            self.root_dir)
        self.assertSetEqual(set(file_path_list), {
                            r_path, a10_path, aa11_path, bb20_path, bb21_path, c30_path})

    def test_get_file_name_list_from_dir(self):
        from zzpy import get_file_name_list_from_dir
        from zzpy import write_file
        from zzpy import init_dir
        import os

        # init
        a_dir = os.path.join(self.root_dir, "a")
        aa_dir = os.path.join(a_dir, "aa")
        init_dir(aa_dir)
        a10_path = os.path.join(a_dir, "10.txt")
        write_file("10.txt", a10_path)
        aa11_path = os.path.join(aa_dir, "11.txt")
        write_file("11.txt", aa11_path)

        b_dir = os.path.join(self.root_dir, "b")
        bb_dir = os.path.join(b_dir, "bb")
        init_dir(bb_dir)
        bb20_path = os.path.join(bb_dir, "20.txt")
        write_file("20.txt", bb20_path)
        bb21_path = os.path.join(bb_dir, "21.txt")
        write_file("21.txt", bb21_path)

        c_dir = os.path.join(self.root_dir, "c")
        init_dir(c_dir)
        c30_path = os.path.join(c_dir, "30.txt")
        write_file("30.txt", c30_path)

        r_path = os.path.join(self.root_dir, "r.txt")
        write_file("r.txt", r_path)

        # test
        file_path_list = get_file_name_list_from_dir(
            self.root_dir)
        self.assertSetEqual(set(file_path_list), {
                            "r.txt", "10.txt", "11.txt", "20.txt", "21.txt", "30.txt"})

    def test_get_file_line_count(self):
        from zzpy import get_file_line_count
        import os

        self.assertEqual(get_file_line_count(
            os.path.join("test", "static", "1.jsonl")), 2)
        self.assertEqual(get_file_line_count(
            os.path.join("test", "static", "2.jsonl")), 2)
        self.assertNotEqual(get_file_line_count(
            os.path.join("test", "static", "3.xls")), 0)
        self.assertNotEqual(get_file_line_count(
            os.path.join("test", "static", "4.xlsx")), 0)

    def test_get_tmp_dir_path(self):
        from zzpy import get_tmp_dir_path
        import os

        self.assertEqual(get_tmp_dir_path(), os.path.join("output", "tmp"))

    def test_get_work_dir_path(self):
        from zzpy import get_work_dir_path
        import os

        self.assertEqual(get_work_dir_path(), os.path.join("output", "work"))

    def test_write_file_and_read_file_when_dir_not_exist(self):
        from zzpy import write_file
        from zzpy import read_file
        from zzpy import read_bin_file
        import os

        write_content = "abc"
        self.assertFalse(os.path.exists(self.root_dir))
        file_path = os.path.join(self.root_dir, f"1.txt")
        write_file(content=write_content, file_path=file_path)
        self.assertTrue(os.path.exists(file_path))
        read_content = read_file(file_path)
        self.assertEqual(read_content, write_content)

    def test_write_file_and_read_file(self):
        from zzpy import write_file
        from zzpy import read_file
        from zzpy import read_bin_file
        import os

        # 校验当文件目录不存在时，是否可以写文件
        self.assertFalse(os.path.exists(self.root_dir))

        for i, write_content in enumerate(("abc", "你好", "abc你好", "你好abc", "こんにちは", "こんにちはzero", "zeroこんにちは", "你好zeroこんにちは", "こんにちはzero你好", "")):
            # default encoding
            file_path = os.path.join(self.root_dir, f"{i}-default.txt")
            write_file(content=write_content, file_path=file_path)
            self.assertTrue(os.path.exists(file_path))
            read_content = read_file(file_path)
            self.assertEqual(read_content, write_content)

            # encoding utf8
            file_path = os.path.join(self.root_dir, f"{i}-utf8.txt")
            write_file(content=write_content,
                       file_path=file_path, encoding="utf8")
            self.assertTrue(os.path.exists(file_path))
            read_content = read_file(file_path)
            self.assertEqual(read_content, write_content)

            # encoding gbk
            file_path = os.path.join(self.root_dir, f"{i}-gbk.txt")
            write_file(content=write_content,
                       file_path=file_path, encoding="gbk")
            self.assertTrue(os.path.exists(file_path))
            read_content = read_file(file_path)
            self.assertEqual(read_content, write_content)

    def test_get_one_file_path_from_dir_when_no_file(self):
        import os
        from zzpy import get_one_file_path_from_dir
        from zzpy import create_dir

        create_dir(os.path.join(self.root_dir, "empty-dir"))
        self.assertIsNone(get_one_file_path_from_dir(
            os.path.join(self.root_dir, "empty-dir")))

    def test_get_one_file_path_from_dir_when_single_file(self):
        import os
        from zzpy import write_file
        from zzpy import get_one_file_path_from_dir

        write_file("CONTENT", os.path.join(self.root_dir, "a", "a1.txt"))
        self.assertEqual(get_one_file_path_from_dir(os.path.join(self.root_dir, "a")), os.path.join(
            self.root_dir, "a", "a1.txt"))

    def test_get_one_file_path_from_dir_when_multi_files(self):
        import os
        from zzpy import write_file
        from zzpy import get_one_file_path_from_dir

        write_file("CONTENT", os.path.join(self.root_dir, "a", "a1.txt"))
        write_file("CONTENT", os.path.join(self.root_dir, "a", "a2.txt"))
        self.assertIn(get_one_file_path_from_dir(os.path.join(self.root_dir, "a")), (os.path.join(
            self.root_dir, "a", "a1.txt"), os.path.join(self.root_dir, "a", "a2.txt")))

    def test_get_one_file_path_from_dir_when_having_dirs_after_file(self):
        import os
        from zzpy import write_file
        from zzpy import get_one_file_path_from_dir

        write_file("CONTENT", os.path.join(self.root_dir, "c", "0.txt"))
        write_file("CONTENT", os.path.join(self.root_dir, "c", "3", "0.txt"))
        write_file("CONTENT", os.path.join(self.root_dir, "c", "1", "0.txt"))
        self.assertTrue(get_one_file_path_from_dir(os.path.join(
            self.root_dir, "c")), os.path.join(self.root_dir, "c", "0.txt"))

    def test_get_one_file_path_from_dir_when_having_dirs_before_file(self):
        import os
        from zzpy import write_file
        from zzpy import get_one_file_path_from_dir

        write_file("CONTENT", os.path.join(self.root_dir, "c", "4.txt"))
        write_file("CONTENT", os.path.join(self.root_dir, "c", "3", "0.txt"))
        write_file("CONTENT", os.path.join(self.root_dir, "c", "1", "0.txt"))
        self.assertTrue(get_one_file_path_from_dir(os.path.join(
            self.root_dir, "c")), os.path.join(self.root_dir, "c", "4.txt"))

    def test_move_file_from_src_dir_when_no_file_in_src(self):
        """当src没有文件时"""
        import os
        from zzpy import write_file
        from zzpy import get_file_path_list_from_dir
        from zzpy import move_file_from_src_dir

        move_file_from_src_dir(dst_file_path=os.path.join(self.root_dir,
                                                          "dst", "1.txt"), src_dir_path=os.path.join(self.root_dir, "src"))
        self.assertListEqual(get_file_path_list_from_dir(
            os.path.join(self.root_dir, "src")), [])
        self.assertListEqual(get_file_path_list_from_dir(
            os.path.join(self.root_dir, "dst")), [])

    def test_move_file_from_src_dir_when_single_in_src_and_dst_not_exist(self):
        """当src只有一个文件，dst不存在时"""
        import os
        from zzpy import write_file
        from zzpy import get_file_path_list_from_dir
        from zzpy import move_file_from_src_dir

        write_file("1", os.path.join(self.root_dir, "src", "1.txt"))
        move_file_from_src_dir(dst_file_path=os.path.join(self.root_dir,
                                                          "dst", "1.txt"), src_dir_path=os.path.join(self.root_dir, "src"))
        self.assertListEqual(get_file_path_list_from_dir(
            os.path.join(self.root_dir, "src")), [])
        self.assertListEqual(get_file_path_list_from_dir(os.path.join(self.root_dir, "dst")), [os.path.join(self.root_dir,
                                                                                                            "dst", "1.txt")])

    def test_move_file_from_src_dir_when_single_in_src_and_dst_empty(self):
        """当src只有一个文件，dst为空文件夹时"""
        import os
        from zzpy import write_file
        from zzpy import get_file_path_list_from_dir
        from zzpy import move_file_from_src_dir
        from zzpy import create_dir

        write_file("1", os.path.join(self.root_dir, "src", "1.txt"))
        create_dir(os.path.join(self.root_dir, "dst"))
        move_file_from_src_dir(dst_file_path=os.path.join(self.root_dir,
                                                          "dst", "1.txt"), src_dir_path=os.path.join(self.root_dir, "src"))
        self.assertListEqual(get_file_path_list_from_dir(
            os.path.join(self.root_dir, "src")), [])
        self.assertListEqual(get_file_path_list_from_dir(os.path.join(self.root_dir, "dst")), [os.path.join(self.root_dir,
                                                                                                            "dst", "1.txt")])

    def test_move_file_from_src_dir_when_single_in_src_and_dst_not_empty(self):
        """当src只有一个文件，dst不为空时"""
        import os
        from zzpy import write_file
        from zzpy import get_file_path_list_from_dir
        from zzpy import move_file_from_src_dir
        from zzpy import create_dir

        write_file("1", os.path.join(self.root_dir, "src", "1.txt"))
        write_file("2", os.path.join(self.root_dir, "dst", "2.txt"))
        move_file_from_src_dir(dst_file_path=os.path.join(self.root_dir,
                                                          "dst", "1.txt"), src_dir_path=os.path.join(self.root_dir, "src"))
        self.assertListEqual(get_file_path_list_from_dir(
            os.path.join(self.root_dir, "src")), [])
        self.assertListEqual(get_file_path_list_from_dir(os.path.join(self.root_dir, "dst")), [os.path.join(self.root_dir,
                                                                                                            "dst", "1.txt"), os.path.join(self.root_dir, "dst", "2.txt")])

    def test_move_file_from_src_dir_when_multi_in_src(self):
        """当src有多个文件时"""
        import os
        from zzpy import write_file
        from zzpy import get_file_path_list_from_dir
        from zzpy import move_file_from_src_dir
        from zzpy import create_dir

        write_file("1", os.path.join(self.root_dir, "src", "1.txt"))
        write_file("2", os.path.join(self.root_dir, "src", "2.txt"))
        write_file("2", os.path.join(self.root_dir, "dst", "3.txt"))
        move_file_from_src_dir(dst_file_path=os.path.join(self.root_dir,
                                                          "dst", "0.txt"), src_dir_path=os.path.join(self.root_dir, "src"))
        src_pathes = get_file_path_list_from_dir(
            os.path.join(self.root_dir, "src"))
        self.assertEqual(len(src_pathes), 1)
        self.assertIn(src_pathes[0], (os.path.join(
            self.root_dir, "src", "1.txt"), os.path.join(self.root_dir, "src", "2.txt")))
        dst_pathes = get_file_path_list_from_dir(
            os.path.join(self.root_dir, "dst"))
        self.assertEqual(len(dst_pathes), 2)
        self.assertListEqual(dst_pathes, [os.path.join(
            self.root_dir, "dst", "0.txt"), os.path.join(self.root_dir, "dst", "3.txt")])

    def test_move_file_from_src_dir_when_src_and_dst_have_same_file(self):
        """当src有多个文件时"""
        import os
        from zzpy import write_file
        from zzpy import get_file_path_list_from_dir
        from zzpy import move_file_from_src_dir
        from zzpy import read_file

        write_file("1", os.path.join(self.root_dir, "src", "1.txt"))
        write_file("2", os.path.join(self.root_dir, "dst", "1.txt"))
        move_file_from_src_dir(dst_file_path=os.path.join(self.root_dir,
                                                          "dst", "1.txt"), src_dir_path=os.path.join(self.root_dir, "src"))
        src_pathes = get_file_path_list_from_dir(
            os.path.join(self.root_dir, "src"))
        self.assertEqual(len(src_pathes), 0)

        dst_pathes = get_file_path_list_from_dir(
            os.path.join(self.root_dir, "dst"))
        self.assertEqual(len(dst_pathes), 1)
        dst_path = dst_pathes[0]
        self.assertEqual(dst_path, os.path.join(self.root_dir,
                                                "dst", "1.txt"))
        self.assertEqual(read_file(dst_path), "1")

    def test_normalize_path(self):
        import os
        from zzpy import windows_else
        from zzpy import normalize_path

        self.assertEqual(normalize_path(""), windows_else("", ""))

        self.assertEqual(normalize_path("."), windows_else(".", "."))
        self.assertEqual(normalize_path("./"), windows_else(".\\", "./"))
        self.assertEqual(normalize_path(".\\"), windows_else(".\\", "./"))
        self.assertEqual(normalize_path("./a"), windows_else(".\\a", "./a"))
        self.assertEqual(normalize_path(".\\a"), windows_else(".\\a", "./a"))

        self.assertEqual(normalize_path(".."), windows_else("..", ".."))
        self.assertEqual(normalize_path("../"), windows_else("..\\", "../"))
        self.assertEqual(normalize_path("..\\"), windows_else("..\\", "../"))
        self.assertEqual(normalize_path("../a"), windows_else("..\\a", "../a"))
        self.assertEqual(normalize_path("..\\a"),
                         windows_else("..\\a", "../a"))

        self.assertEqual(normalize_path("E:/a.txt"),
                         windows_else("E:a.txt", "E:/a.txt"))
        self.assertEqual(normalize_path("E:\\a.txt"),
                         windows_else("E:a.txt", "E:/a.txt"))

        self.assertEqual(normalize_path("E:/x/a.txt"),
                         windows_else("E:x\\a.txt", "E:/x/a.txt"))
        self.assertEqual(normalize_path("E:\\x\\a.txt"),
                         windows_else("E:x\\a.txt", "E:/x/a.txt"))

        self.assertEqual(normalize_path("/a.txt"),
                         windows_else("/a.txt", "/a.txt"))
        self.assertEqual(normalize_path("/x/a.txt"),
                         windows_else("/x\\a.txt", "/x/a.txt"))



def fix_gbk(object):
    """
    修复gbk编码问题
    'gbk' codec can't encode character '\xa0' in position 202: illegal multibyte sequence
    """
    return object.replace('\xa0', '') if isinstance(object, str) else object


def trans_excel_to_csv(excel_path, csv_path, encoding="utf8", gbk_fixing=True):
    import csv
    import openpyxl
    with open(csv_path, mode="w", newline='', encoding=encoding) as fw:
        writer = csv.writer(fw, delimiter=',')
        wb = openpyxl.load_workbook(excel_path)
        ws = wb.active
        for row in ws.rows:
            if gbk_fixing:
                writer.writerow([fix_gbk(c.value) for c in row])
            else:
                writer.writerow([c.value for c in row])


def read_csv_dict(path):
    import csv
    for e in ("utf8", "gbk"):
        try:
            with open(path, encoding=e) as fr:
                reader = csv.reader(fr)
                head = next(reader)
                for row in reader:
                    yield dict(zip(head, row))
            return
        except:
            pass
    error_msg = f"文件编码错误: {path}"
    raise Exception(error_msg)


def read_csv_head(path):
    import csv
    for e in ("utf8", "gbk"):
        try:
            with open(path, encoding=e) as fr:
                reader = csv.reader(fr)
                return next(reader)
        except:
            pass
    error_msg = f"文件编码错误: {path}"
    raise Exception(error_msg)


def read_csv_rows(path):
    import csv
    for e in ("utf8", "gbk"):
        try:
            with open(path, encoding=e) as fr:
                reader = csv.reader(fr)
                next(reader)
                yield from reader
            return
        except:
            pass
    error_msg = f"文件编码错误: {path}"
    raise Exception(error_msg)


# def save_data_to_excel(data, excel_path, skip_error_row=False):
#     import openpyxl
#     wb = openpyxl.Workbook()
#     ws = wb.active
#     for row in data:
#         try:
#             ws.append(row)
#         except Exception as ex:
#             if not skip_error_row:
#                 raise ex
#     wb.save(excel_path)


# def read_xls(filepath, sheetname=None):
#     import xlrd
#     from datetime import datetime
#     rbook = xlrd.open_workbook(filepath)
#     if sheetname:
#         sheet = rbook.sheet_by_name(sheetname)
#     else:
#         sheet = rbook.sheets()[0]
#     rows = sheet.nrows
#     cols = sheet.ncols
#     all_content = []
#     for i in range(rows):
#         row_content = []
#         for j in range(cols):
#             ctype = sheet.cell(i, j).ctype  # 表格的数据类型
#             cell = sheet.cell_value(i, j)
#             if ctype == 2 and cell % 1 == 0:  # 如果是整形
#                 cell = int(cell)
#             elif ctype == 3:
#                 # 转成datetime对象
#                 date = datetime(*xlrd.xldate_as_tuple(cell, 0))
#                 cell = date.strftime('%Y-%m-%d %H:%M:%S')
#             elif ctype == 4:
#                 cell = True if cell == 1 else False
#             row_content.append(cell)
#         all_content.append(row_content)
#         # print('[' + ','.join("'" + str(element) + "'" for element in row_content) + ']')
#     return all_content


# def get_data_from_excel(excel_path):
#     if excel_path.endswith(".xlsx"):
#         import openpyxl
#         wb = openpyxl.load_workbook(excel_path)
#         ws = wb.active
#         return list(ws.values)
#     else:
#         return read_xls(excel_path)


# def get_data_from_excel_streamingly(filepath):
#     if filepath.endswith(".xlsx"):
#         import openpyxl
#         wb = openpyxl.load_workbook(filepath)
#         ws = wb.active
#         for r in ws.values:
#             yield r
#     else:
#         import xlrd
#         from datetime import datetime
#         rbook = xlrd.open_workbook(filepath)
#         sheet = rbook.sheets()[0]
#         rows = sheet.nrows
#         cols = sheet.ncols
#         for i in range(rows):
#             row_content = []
#             for j in range(cols):
#                 ctype = sheet.cell(i, j).ctype  # 表格的数据类型
#                 cell = sheet.cell_value(i, j)
#                 if ctype == 2 and cell % 1 == 0:  # 如果是整形
#                     cell = int(cell)
#                 elif ctype == 3:
#                     # 转成datetime对象
#                     date = datetime(*xlrd.xldate_as_tuple(cell, 0))
#                     cell = date.strftime('%Y-%m-%d %H:%M:%S')
#                 elif ctype == 4:
#                     cell = True if cell == 1 else False
#                 row_content.append(cell)
#             yield row_content



import re
import oss2

__OSS_URL_KEY = "OSS_URL"


class OssConfig:
    def __init__(self, url):
        result = re.search("^(oss://){0,1}([^/]+)/([^?]+)\?(.*)$", url)
        groups = result.groups()
        endpoint = f"https://{groups[1]}"
        assert endpoint
        bucket = groups[2]
        assert bucket
        param = dict([it.split('=')
                      for it in groups[3].split('&')])
        assert param
        access_key_id = param.get("key")
        assert access_key_id
        access_key_secret = param.get("secret")
        assert access_key_secret

        self.access_key_id = access_key_id
        self.access_key_secret = access_key_secret
        self.bucket = bucket
        self.endpoint = endpoint


def oss_connect(url=None):
    if not url:
        from .zconfig import get_param
        url = get_param(__OSS_URL_KEY)
    assert url

    return AliOss(url)


class AliOss:
    def __init__(self, url):
        config = OssConfig(url)
        self.config = config
        self.bucket = oss2.Bucket(
            oss2.Auth(config.access_key_id, config.access_key_secret), config.endpoint, config.bucket)
        
    def url(self, key):
        return f"{self.config.bucket}/{key}"

    def download(self, key, file_path):
        """下载文件"""
        return self.bucket.get_object_to_file(key, file_path)

    def upload(self, key, file_path, infrequent_access_flag=True):
        """上传文件，infrequent_access_flag低频标识"""
        import oss2
        headers = {
            'x-oss-storage-class': oss2.BUCKET_STORAGE_CLASS_IA} if infrequent_access_flag else None
        return self.bucket.put_object_from_file(key, file_path)

    def list(self, prefix):
        next_marker = ""
        while True:
            result = self.bucket.list_objects(
                prefix=prefix, max_keys=1_000, marker=next_marker)
            for it in result.object_list:
                yield it.key
            next_marker = result.next_marker
            if not next_marker:
                break

    def delete(self, prefix):
        for key in self.list():
            self.bucket.delete_object(key=key)

    def exist(self, key):
        """判断文件是否已存在"""
        return self.bucket.object_exists(key)

    def not_exist(self, key):
        """判断文件是否不存在"""
        return not self.exist(key)

    def meta(self, key):
        """获取文件元信息"""
        return self.bucket.get_object_meta(key)



#!/usr/bin/env python
# -*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: Zero
# Mail: zhumavip@163.com
# Created Time:  2020-07-06
#############################################

from setuptools import setup, find_packages

setup(
    name="zzpy",  # 这里是pip项目发布的名称
    version="1.0.20200825.10",  # 版本号，数值大的会优先被pip
    keywords=["pip", "zzpy", "utils"],
    description="Python3 utilities",
    long_description="Python3 utilities",
    license="MIT Licence",

    url="https://github.com/zerocpp/zzpy",  # 项目相关文件地址，一般是github
    author="zerocpp",
    author_email="85642468@qq.com",

    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=["pymongo", "pymysql",
                      "redis", "tqdm", "jsonlines", "pandas", "oss2", "openpyxl"]  # 这个项目需要的第三方库
)



import zzpy as z
import os
import unittest
import warnings


class TestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        warnings.simplefilter('ignore', ResourceWarning)

    def setUp(self):
        import os
        super().setUp()
        self.root_dir = os.path.join("test", "file")
        from zzpy import init_dir
        init_dir(self.root_dir)

    def tearDown(self):
        from zzpy import remove_dir
        super().tearDown()
        remove_dir(self.root_dir)

    def test_alioss(self):
        oss = z.AliOss("oss://oss-cn-zhangjiakou.aliyuncs.com/data-3rd?key=LTAI4Fr9r2aYm6nLxGGzfWU9&secret=alEzYCJdyqBHLIobHjFHkZXstvxMEn")
        a_path = "setup.py"
        b_path = os.path.join(self.root_dir, "b.py")
        oss.upload("test/setup.py", a_path)
        oss.download("test/setup.py", b_path)
        a = z.read_file(a_path)
        b = z.read_file(b_path)
        self.assertEqual(a, b)
        
    def test_alioss_with_config_file(self):
        oss = z.AliOss("oss://oss-cn-zhangjiakou.aliyuncs.com/data-3rd?key=LTAI4Fr9r2aYm6nLxGGzfWU9&secret=alEzYCJdyqBHLIobHjFHkZXstvxMEn")
        a_path = "setup.py"
        b_path = os.path.join(self.root_dir, "b.py")
        oss.upload(key="test/setup1.py", file_path=a_path)
        oss.download(key="test/setup1.py", file_path=b_path)
        a = z.read_file(a_path)
        b = z.read_file(b_path)
        self.assertEqual(a, b)



import unittest
from zzpy import randhex
import re


class ZRandomRandhexTests(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_randhex_when_no_parameters(self):
        for _ in range(10):
            n = randhex()
            self.assertIsInstance(n, str)
            self.assertEqual(len(n), 1)
            self.assertIsNotNone(re.match("^[\dA-F]$", n))

    def test_randhex_when_digits_set(self):
        for digits in range(1, 100):
            n = randhex(digits=digits)
            self.assertIsInstance(n, str)
            self.assertEqual(len(n), digits)
            self.assertIsNotNone(re.match(f"^[\dA-F]{{{digits}}}$", n))

    def test_randhex_when_digits_set_0(self):
        n = randhex(digits=0)
        self.assertIsInstance(n, str)
        self.assertEqual(n, "")

    def test_randhex_when_prefix_set_true(self):
        for digits in range(1, 10):
            n = randhex(digits=digits, prefix=True)
            self.assertIsInstance(n, str)
            self.assertEqual(len(n), 2+digits)
            self.assertIsNotNone(re.match(f"^0x[\dA-F]{{{digits}}}$", n))

    def test_randhex_when_prefix_set_false(self):
        for digits in range(1, 10):
            n = randhex(digits=digits, prefix=False)
            self.assertIsInstance(n, str)
            self.assertEqual(len(n), digits)
            self.assertIsNotNone(re.match(f"^[\dA-F]{{{digits}}}$", n))

    def test_randhex_when_lower_set_true(self):
        digits = 1_000
        self.assertIsNotNone(
            re.match(f"^[\da-f]{{{digits}}}$", randhex(digits=digits, lower=True)))
        self.assertIsNotNone(
            re.match(f"^[\da-f]{{{digits}}}$", randhex(digits=digits, prefix=False, lower=True)))
        self.assertIsNotNone(re.match(
            f"^0x[\da-f]{{{digits}}}$", randhex(digits=digits, prefix=True, lower=True)))

    def test_randhex_when_lower_set_false(self):
        digits = 1_000
        self.assertIsNotNone(
            re.match(f"^[\dA-F]{{{digits}}}$", randhex(digits=digits)))
        self.assertIsNotNone(
            re.match(f"^[\dA-F]{{{digits}}}$", randhex(digits=digits, lower=False)))
        self.assertIsNotNone(
            re.match(f"^[\dA-F]{{{digits}}}$", randhex(digits=digits, prefix=False, lower=False)))
        self.assertIsNotNone(re.match(
            f"^0x[\dA-F]{{{digits}}}$", randhex(digits=digits, prefix=True, lower=False)))



__MYSQL_URL_KEY = "MYSQL_URL"


class MySQLConfig:
    default_host = "localhost"
    default_port = 3306

    def __init__(self, url=None, host=None, port=None, database=None, param=None, user=None, password=None):
        import re
        if url:
            result = re.search(
                "^(mysql://){0,1}([^:/]+)(:(\d+)){0,1}(/([^?]*)){0,1}(\?(.*)){0,1}", url)
            groups = result.groups()
            if not host:
                host = groups[1] if groups[1] else self.default_host
            if not port:
                port = int(groups[3]) if groups[3] else self.default_port
            if not database:
                database = groups[5] if groups[5] else None
            if not param:
                param = dict([it.split('=')
                              for it in groups[7].split('&')]) if groups[7] else {}
            if not user:
                user = param.get("user")
            if not password:
                password = param.get("password")

        self.url = url
        self.host = host
        self.port = port
        self.database = database
        self.param = param if param else {}
        self.user = user
        self.password = password

    def to_dict(self):
        d = {}
        if self.url is not None:
            d["url"] = self.url
        if self.host is not None:
            d["host"] = self.host
        if self.port is not None:
            d["port"] = self.port
        if self.database is not None:
            d["database"] = self.database
        if self.param is not None:
            d["param"] = self.param
        if self.user is not None:
            d["user"] = self.user
        if self.password is not None:
            d["password"] = self.password
        return d

    def __eq__(self, value):
        return self.host == value.host and self.port == value.port and self.database == value.database and self.param == value.param and self.user == value.user and self.password == value.password


def mysql_connect(url=None, autocommit=True):
    if not url:
        from .zconfig import get_param
        url = get_param(__MYSQL_URL_KEY)
    assert url

    import pymysql
    conf = MySQLConfig(url)

    client = pymysql.Connect(
        host=conf.host, port=conf.port, database=conf.database, user=conf.user, password=conf.password)
    client.autocommit(autocommit)
    return client


def mysql_query_one_value(client, sql):
    cursor = client.cursor()
    cursor.execute(sql)
    return cursor.fetchall()[0][0]


def mysql_query(client, sql):
    cursor = client.cursor()
    cursor.execute(sql)
    return cursor.fetchall()


def mysql_execute(client, sql):
    cursor = client.cursor()
    return cursor.execute(sql)


def mysql_insert(client, sql):
    cursor = client.cursor()
    res = cursor.execute(sql)
    return cursor.lastrowid if res else 0


def mysql_iter_table(client, table, fields=None, where_condition=None, offset_limit=None):
    if where_condition:
        if not where_condition.startswith("where") and not where_condition.startswith("WHERE"):
            where_condition = "where "+where_condition
    else:
        where_condition = ""
    if not offset_limit:
        offset_limit = ""
    if fields:
        fields = ",".join(f"`{i}`" for i in fields)
    else:
        fields = "*"
    sql = f"select {fields} from {table} {where_condition} {offset_limit}"
    from pymysql.cursors import SSDictCursor
    cursor = SSDictCursor(client)
    cursor.execute(sql)
    while True:
        item = cursor.fetchone()
        if not item:
            cursor.close()
            return
        yield item


def mysql_count_table(client, table, where_condition=None, offset_limit=None):
    if where_condition:
        if not where_condition.startswith("where") and not where_condition.startswith("WHERE"):
            where_condition = "where "+where_condition
    else:
        where_condition = ""
    if not offset_limit:
        offset_limit = ""
    sql = f"select count(*) from {table} {where_condition} {offset_limit}"
    return mysql_query_one_value(client, sql)


class ZMySQL:
    def __init__(self, url=None):
        self.client = mysql_connect(url=url)

    def execute(self, sql):
        return mysql_execute(self.client, sql)

    def insert(self, sql):
        return mysql_insert(self.client, sql)

    def query(self, sql):
        return mysql_query(self.client, sql)

    def query_one_value(self, sql):
        return mysql_query_one_value(self.client, sql)
